#!/bin/bash
#
# Writen by Dawid Jagieła and Adam Sieradzan
# and Cezary Czaplewski
#
#
#----------------------------------------
# variables

if [[ "$2" =~ "MPI" ]]; then
 file=$1.out_GB000
 file_stat=$1_GB000.stat
else
 file=$1.out_GB					# Output file to search energy value from
 file_stat=$1_GB.stat
fi

if [ "$1" == "prota_ENE_m" ]; then
 extremediff="10000.0"			# extreme energy difference, comething went terribly wrong
 expectenergy="146.0091"	# - expected total energy
 cutoffdiff="0.01"			# energy cutoff variation - more then this rises warning  
elif [ "$1" == "prota_MIN_CART" ]; then
 extremediff="10.0"			# extreme energy difference, comething went terribly wrong
 expectenergy="158.7321"		# - expected total energy
 cutoffdiff="0.1"			# energy cutoff variation - more then this rises warning  
 
 sumsl_return=`grep SUMSL $file|awk '{print $4}'`
 echo 'SUMSL return code' $sumsl_return
 if [ "$sumsl_return" != "4" ]; then
   echo 'ERROR = SUMSL return code' $sumsl_return 'is not 4'
#   exit 1
 fi
elif [ "$1" == "1l2y_MIN_INT" ]; then
 extremediff="10.0"			# extreme energy difference, comething went terribly wrong
 expectenergy="-70.33"			# expected total energy
 cutoffdiff="0.1"			# energy cutoff variation - more then this rises warning  
 
 sumsl_return=`grep SUMSL $file|awk '{print $4}'`
 echo 'SUMSL return code' $sumsl_return
 if [ "$sumsl_return" != "4" ]; then
   echo 'ERROR = SUMSL return code' $sumsl_return 'is not 4'
#   exit 1
 fi
elif [ "$1" == "1l2y_MIN_REGULAR_INT" ]; then
 extremediff="10.0"			# extreme energy difference, comething went terribly wrong
 expectenergy="-6.55"			# expected total energy
#
# something wrong with REGULAR and sometimes gives code 8 and different energy
# for now 8 is only warning and cutoffdiff is large
#
 cutoffdiff="6.0"			# energy cutoff variation - more then this rises warning  
 
 sumsl_return=`grep "SUMSL return code:" $file|awk '{print $4}'`
 echo 'SUMSL return code' $sumsl_return
 if [ "$sumsl_return" != "4" ]; then
   echo 'WARNING = SUMSL return code' $sumsl_return 'is not 4'
 fi

elif [ "$1" == "1l2y_micro" ]; then
 refe="74.2623"
 stat=`awk '{if ( $1 != "#" ) {n++;a=a+$5;a2=a2+$5^2}}END{print a/n,sqrt((a2-a^2/n)/n)}' $file_stat`
 array=(${stat// / })
 echo 'average total energy ' ${array[0]}
 echo 'standard deviation ' ${array[1]}
 if [ `echo "a=${array[0]}-${refe};if(0>a)a*=-1;a>5.0"|bc -l` != "0" ]; then
  echo 'difference ' `echo "a=${array[0]}-${refe};if(0>a)a*=-1;a"|bc -l` "from reference ave etot ${refe} greater than 5.0"
  exit 1
 elif [ `echo "a=${array[1]};a>0.30"|bc -l` != "0" ]; then
  echo 'standard deviation greater than 0.30'
  exit 1
 else
  exit 0
 fi
elif [ "$1" == "1L2Y_L" ] || [ "$1" == "1L2Y_NH" ]; then
 if [ ! -s $file_stat ]; then
  echo 'FATAL error - stat file empty'
  exit 2
 fi
 chi2=`./matplotlib_fit_hist.py $file_stat 111`
 echo 'Chi2 for fitting theoretical temperature distribution ' ${chi2}
 echo  "<DartMeasurementFile name=\"Temperature distribution $1\" type=\"image/png\">`pwd`/${file_stat}.png</DartMeasurementFile>"
 
 if [ `echo "a=${chi2};a>0.001"|bc -l` != "0" ]; then
    echo 'chi2 greater than 0.001'
    exit 1
 else
    exit 0
 fi
elif [ "$1" == "1L2Y_B" ]; then
 if [ ! -s $file_stat ]; then
  echo 'FATAL error - stat file empty'
  exit 2
 fi
 chi2=`./matplotlib_fit_hist.py $file_stat 111`
 echo 'Chi2 for fitting theoretical temperature distribution ' ${chi2}
 echo  "<DartMeasurementFile name=\"Temperature distribution $1\" type=\"image/png\">`pwd`/${file_stat}.png</DartMeasurementFile>"
 
 if [ `echo "a=${chi2};a>0.01"|bc -l` != "0" ]; then
    echo 'chi2 greater than 0.01'
    exit 1
 else
    exit 0
 fi


elif [ "$1" == "1L2Y_remd" ]; then
 rm -rf remd_all.stat
 tail -q -n +1000 1L2Y_remd*.stat >remd_all.stat
 if [ ! -s remd_all.stat ]; then
  echo 'FATAL error - stat files empty'
  exit 2
 fi
 ./matplotlib_hist.py

 echo  "<DartMeasurementFile name=\"Histograms $1\" type=\"image/png\">`pwd`/1L2Y_remd_ene_hist.png</DartMeasurementFile>"
 echo  "<DartMeasurementFile name=\"Energy $1\" type=\"image/png\">`pwd`/1L2Y_remd_Tene.png</DartMeasurementFile>" 
 grep ACC 1L2Y_remd.out_GB000 |tail -7
 exchange=`grep ACC 1L2Y_remd.out_GB000 |tail -7|awk '{a=a+$4}END{print a/NR}'`
 echo "average exchange = ${exchange}"
 if [ `echo "a=${exchange};a<0.2"|bc -l` != "0" ]; then
    echo 'ERROR average exchange smaller than 0.2'
    exit 1
 else
    exit 0
 fi

elif [ "$1" == "1DKZcut-ber" ]; then
 chi2=`./matplotlib_fit_hist.py $file_stat 519`
 echo 'Chi2 for fitting theoretical temperature distribution ' ${chi2}
 echo  "<DartMeasurementFile name=\"Temperature distribution $1\" type=\"image/png\">`pwd`/${file_stat}.png</DartMeasurementFile>"
 
 if [ `echo "a=${chi2};a>0.01"|bc -l` != "0" ]; then
    echo 'chi2 greater than 0.01'
    exit 1
 else
    exit 0
 fi

elif [ "$1" == "1DKZcut-lang" ]; then
 chi2=`./matplotlib_fit_hist.py $file_stat 519`
 echo 'Chi2 for fitting theoretical temperature distribution ' ${chi2}
 echo  "<DartMeasurementFile name=\"Temperature distribution $1\" type=\"image/png\">`pwd`/${file_stat}.png</DartMeasurementFile>"
 
 if [ `echo "a=${chi2};a>0.001"|bc -l` != "0" ]; then
    echo 'chi2 greater than 0.001'
    exit 1
 else
    exit 0
 fi

elif [ "$1" == "1DKZcut-min" ]; then
 extremediff="10.0"			# extreme energy difference, comething went terribly wrong
 expectenergy="323.6931"		# - expected total energy
 cutoffdiff="8.0"			# energy cutoff variation - more then this rises warning  
 
 sumsl_return=`grep SUMSL $file|awk '{print $4}'`
 echo 'SUMSL return code' $sumsl_return
 if [ "$sumsl_return" != "4" ]; then
   echo 'ERROR = SUMSL return code' $sumsl_return 'is not 4'
   echo 'but not failing this test, it is known problem with esccor'
#   exit 1
 fi

elif [ "$1" == "1DKZcut-ene" ]; then
 extremediff="10000.0"			# extreme energy difference, comething went terribly wrong
 expectenergy="-443.2674"	# - expected total energy
 cutoffdiff="0.01"			# energy cutoff variation - more then this rises warning  

elif [ "$1" == "1DKZcut-micro" ]; then
 refe="-173.573"
 stat=`awk '{if ( $1 != "#" ) {n++;a=a+$5;a2=a2+$5^2}}END{print a/n,sqrt((a2-a^2/n)/n)}' $file_stat`
 array=(${stat// / })
 echo 'average total energy ' ${array[0]}
 echo 'standard deviation ' ${array[1]}
 if [ `echo "a=${array[0]}-(${refe});if(0>a)a*=-1;a>5.0"|bc -l` != "0" ]; then
  echo 'difference ' `echo "a=${array[0]}-${refe};if(0>a)a*=-1;a"|bc -l` "from reference ave etot ${refe} greater than 5.0"
  exit 1
 elif [ `echo "a=${array[1]};a>0.005"|bc -l` != "0" ]; then
  echo 'standard deviation greater than 0.005'
  exit 1
 else
  exit 0
 fi

elif [ "$1" == "1ei0_min" ]; then
 extremediff="20.0"			# extreme energy difference, comething went terribly wrong
 expectenergy="141.9976"		# - expected total energy
 cutoffdiff="10.0"			# energy cutoff variation - more then this rises warning  
 
 refe="134.706"
 startene=`grep ETOT $file|head -1| awk '{print $2*1.0}'`
 echo "initial energy=${startene} reference=${refe}"
 if [ `echo "a=${startene}-(${refe});if(0>a)a*=-1;a>0.01"|bc -l` != "0" ]; then
  echo 'difference ' `echo "a=${startene}-${refe};if(0>a)a*=-1;a"|bc -l` "from reference etot ${refe} greater than 0.01"
  exit 1
 fi
 
 sumsl_return=`grep SUMSL $file|awk '{print $4}'`
 echo 'SUMSL return code' $sumsl_return
 if [ "$sumsl_return" != "4" ]; then
   echo 'ERROR = SUMSL return code' $sumsl_return 'is not 4'
   echo 'but not failing this test, it is known problem with dissulfides'
#   exit 1
 fi

elif [ "$1" == "prota_CHECKGRAD" ] || [ "$1" == "1DKZcut-checkgrad" ] || [ "$1" ==  "checkgrad_dfa" ]  ; then
 diff=`gawk -f checkgrad.awk $file |grep 'Largest abs(1-numerical/analytical)='|awk '{printf "%15.10f",$3}'`
 gawk -f checkgrad.awk $file 
 
 if [ `echo "a=${diff};a>0.025"|bc -l` != "0" ]; then
    echo 'ERROR largest abs(1-numerical/analytical)' ${diff}
    echo ' greater than 0.025'
    exit 1
 else
    exit 0
 fi

elif [ "$1" ==  "Ts866_checkgrad_full" ] ; then
 diff=`gawk -f checkgrad.awk $file |grep 'Largest abs(1-numerical/analytical)='|awk '{printf "%15.10f",$3}'`
 gawk -f checkgrad.awk $file 

 exit_error=0
 ene=`grep ETOT $file|head -1| awk '{print $2*1.0}'`
 hene=`grep H_CONS $file|head -1| awk '{print $2*1.0}'`
 
 ene_ref=3124.347
 hene_ref=1872.981
 
 echo "ETOT= " ${ene} " reference= " ${ene_ref}
 if [ `echo "a=${ene}-(${ene_ref});if(0>a)a*=-1;a>0.01"|bc -l` != "0" ]; then
  echo 'difference ' `echo "a=${ene}-${ene_ref};if(0>a)a*=-1;a"|bc -l` "from reference etot ${ene_ref} greater than 0.01"
  exit_error=1
 fi

 echo "H_CONS= " ${hene} " reference= " ${hene_ref}
 if [ `echo "a=${hene}-(${hene_ref});if(0>a)a*=-1;a>0.01"|bc -l` != "0" ]; then
  echo 'difference ' `echo "a=${hene}-${hene_ref};if(0>a)a*=-1;a"|bc -l` "from reference etot ${hene_ref} greater than 0.01"
  exit_error=1
 fi
 
 
 if [ `echo "a=${diff};a>0.0020"|bc -l` != "0" ]; then
    echo 'ERROR largest abs(1-numerical/analytical)' ${diff}
    echo ' greater than 0.0020'
    exit_error=1
 fi

 if [ "${exit_error}" == "1" ]; then
    exit 1
 else
    exit 0
 fi


elif [ "$1" == "remd_dfa" ]; then
 rm -rf remd_all.stat
 tail -q -n +100 remd_dfa*.stat >remd_all.stat
 if [ ! -s remd_all.stat ]; then
  echo 'FATAL error - stat files empty'
  exit 2
 fi
 ./matplotlib_hist.py

 echo  "<DartMeasurementFile name=\"Histograms $1\" type=\"image/png\">`pwd`/1L2Y_remd_ene_hist.png</DartMeasurementFile>"
 echo  "<DartMeasurementFile name=\"Energy $1\" type=\"image/png\">`pwd`/1L2Y_remd_Tene.png</DartMeasurementFile>" 
 grep ACC remd_dfa.out_GB000 |tail -7
 exchange=`grep ACC remd_dfa.out_GB000 |tail -7|awk '{a=a+$4}END{print a/NR}'`
 echo "average exchange = ${exchange}"
 if [ `echo "a=${exchange};a<0.1"|bc -l` != "0" ]; then
    echo 'ERROR average exchange smaller than 0.1'
    exit 1
 else
    exit 0
 fi

               
else
 exit 1
fi



function floating(){
    echo $1 | sed -e 's/[eE]+/*10^/'
}

#---------------------------------------------------------------------------
# MAIN CODE
#---------------------------------------------------------------------------

# Check if file exist
if [ ! -f $file ]; then
    echo "CRITICAL: out file do not exist"
    exit 2
fi

# Check if energy value is not a number
grepene=`grep ETOT $file|tail -1| awk '{print $2}'`
if [[ $grepene == "NaN" ]]; then
    echo "CRITICAL: energy is NaN"
    exit 2
fi

# Check if energy value is extremely different from expected
blaene=`floating $grepene`
myene=`echo $blaene |bc`
diff=`echo $myene"+("$expectenergy")"|bc`
absdiff=`echo $diff| sed 's/-//'`

true1=`echo "$absdiff >= $extremediff"|bc`
if [ $true1 -eq 1 ]; then
    echo "CRITICAL: energy is loo large $absdiff, ene= $myene"
    exit 2
fi

# Check if Energy value is as expected
true2=`echo "$absdiff<=$cutoffdiff"|bc`
if [ $true2 -eq 1 ]; then
    echo "OK: absdiff= $absdiff, ene= $myene"
    exit 0
else
    echo "WARNING: energy is somewhat different $absdiff, ene= $myene"
    exit 1
fi

